library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;  -- Use OR_REDUCE function

package MTriggerPKG is

	constant V_REVISION		: std_logic_vector(15 downto 0) := x"0100";	-- 1.0
	constant V_MTRIGGERID	: std_logic_vector(6 downto 0) := "1001101";

	type STD_LOGIC_VEC_ARRAY9_37 is array(0 to 36) of std_logic_vector(8 downto 0);	
	type STD_LOGIC_VEC_ARRAY4_37 is array(0 to 36) of std_logic_vector(3 downto 0);
	
	type STD_LOGIC_VEC_ARRAY4 is array(105 downto 0) of std_logic_vector(3 downto 0);
	type STD_LOGIC_VEC_ARRAY9 is array(105 downto 0) of std_logic_vector(8 downto 0);

	function stdvec_bitcnt (b:std_logic_vector; w:integer) return std_logic_vector;	
	
	function sum_9bits(I:std_logic_vector) return std_logic_vector;
	
end MTriggerPKG;

package body MTriggerPKG is

	-- sum_9bits coded for 4-LUT synthesis efficiency...
	function sum_9bits(I:std_logic_vector) return std_logic_vector is
		variable I1, I2, O	: std_logic_vector(3 downto 0);
		variable O1, O2		: std_logic_vector(2 downto 0);
	begin
		O1 := ("00"&I(0 downto 0))+("00"&I(1 downto 1))+("00"&I(2 downto 2))+("00"&I(3 downto 3));
		O2 := ("00"&I(4 downto 4))+("00"&I(5 downto 5))+("00"&I(6 downto 6))+("00"&I(7 downto 7));
		O := ("0"&O1)+("0"&O2)+("000"&I(8 downto 8));
		return O;
	end;

	function stdvec_bitcnt (b:std_logic_vector; w:integer) return std_logic_vector is
		variable result	: std_logic_vector(w-1 downto 0);
	begin
		result := (others => '0');
		for i in b'RANGE loop
			if b(i) = '1' then
				result := result + 1;
			end if;
		end loop;
		return result;
	end;

end MTriggerPKG;