library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

entity ParallelComRx is
	port(
			CLK			: in std_logic;
			RESET		: in std_logic;
			
			DIN			: in std_logic_vector(31 downto 0);
						
			HITBITS		: out std_logic_vector(54 downto 0);
			CLUSTERS	: out std_logic_vector(18 downto 0);
			CLUSTERCNT1	: out std_logic_vector(3 downto 0);
			CLUSTERCNT2	: out std_logic_vector(3 downto 0);
			
			COMERROR	: out std_logic;

			SYNC		: in std_logic
	);
end ParallelComRx;

architecture RTL of ParallelComRx is
	signal DIN_Q				: std_logic_vector(31 downto 0);
	
	signal MSG0_D, MSG0_Q		: std_logic_vector(31 downto 0);
	signal MSG1_D, MSG1_Q		: std_logic_vector(31 downto 0);
	signal MSG2_D, MSG2_Q		: std_logic_vector(31 downto 0);
	
	signal SYNC_Q1, SYNC_Q2		: std_logic;
	
	signal MSG0_ERROR			: std_logic;
	signal MSG1_ERROR			: std_logic;
	signal MSG2_ERROR			: std_logic;
begin

	HITBITS <= MSG1_Q(27 downto 0) & MSG0_Q(26 downto 0);
	CLUSTERS <= MSG2_Q(18 downto 0);
	CLUSTERCNT1 <= MSG2_Q(22 downto 19);
	CLUSTERCNT2 <= MSG2_Q(26 downto 23);
	
	MSG0_D <= DIN_Q when SYNC_Q2 = '1' else MSG0_Q;
	MSG1_D <= DIN_Q when SYNC    = '1' else MSG1_Q;
	MSG2_D <= DIN_Q when SYNC_Q1 = '1' else MSG2_Q;

	MSG0_ERROR <= '1' when MSG0_Q(31 downto 27) /= "11110" else '0';
	MSG1_ERROR <= '1' when MSG1_Q(31 downto 28) /= "1110" else '0';
	MSG2_ERROR <= '1' when MSG2_Q(31 downto 27) /= "10101" else '0';
	
	COMERROR <= MSG0_ERROR or MSG1_ERROR or MSG2_ERROR;

	process(RESET, CLK)
	begin
		if RESET = '1' then
			DIN_Q <= (others => '0');
			SYNC_Q1 <= '0';
			SYNC_Q2 <= '0';
		elsif rising_edge(CLK) then
			DIN_Q <= DIN;
			SYNC_Q1 <= SYNC;
			SYNC_Q2 <= SYNC_Q1;
			MSG0_Q <= MSG0_D;
			MSG1_Q <= MSG1_D;
			MSG2_Q <= MSG2_D;
		end if;
	end process;
	
end RTL;