library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

use work.MTriggerPKG.all;

entity TriggerDetect is
	port(
			CLK				: in std_logic;
			RESET			: in std_logic;

			SYNC			: in std_logic;
			
			Q1_CLUSTERCNT1	: in std_logic_vector(3 downto 0);
			Q1_CLUSTERCNT2	: in std_logic_vector(3 downto 0);
			Q1_CLUSTERS		: in std_logic_vector(36 downto 0);

			Q2_CLUSTERCNT1	: in std_logic_vector(3 downto 0);
			Q2_CLUSTERCNT2	: in std_logic_vector(3 downto 0);
			Q2_CLUSTERS		: in std_logic_vector(36 downto 0);

			Q3_CLUSTERCNT1	: in std_logic_vector(3 downto 0);
			Q3_CLUSTERCNT2	: in std_logic_vector(3 downto 0);
			Q3_CLUSTERS		: in std_logic_vector(36 downto 0);

			Q4_CLUSTERCNT1	: in std_logic_vector(3 downto 0);
			Q4_CLUSTERCNT2	: in std_logic_vector(3 downto 0);
			Q4_CLUSTERS		: in std_logic_vector(36 downto 0);
			
			CLUSTERCNT_THR	: in std_logic_vector(6 downto 0);
			TRIGGER			: out std_logic
	);
end TriggerDetect;

architecture RTL of TriggerDetect is
	signal SYNC_Q1, SYNC_Q2		: std_logic;
	
	signal Q1_CLUSTERCNTA_D		: std_logic_vector(4 downto 0);
	signal Q2_CLUSTERCNTA_D		: std_logic_vector(4 downto 0);
	signal Q3_CLUSTERCNTA_D		: std_logic_vector(4 downto 0);
	signal Q4_CLUSTERCNTA_D		: std_logic_vector(4 downto 0);
	
	signal Q1_CLUSTERCNTA_Q		: std_logic_vector(4 downto 0);
	signal Q2_CLUSTERCNTA_Q		: std_logic_vector(4 downto 0);
	signal Q3_CLUSTERCNTA_Q		: std_logic_vector(4 downto 0);
	signal Q4_CLUSTERCNTA_Q		: std_logic_vector(4 downto 0);

	signal Q1_CLUSTERCNTB_D		: std_logic_vector(4 downto 0);
	signal Q2_CLUSTERCNTB_D		: std_logic_vector(4 downto 0);
	signal Q3_CLUSTERCNTB_D		: std_logic_vector(4 downto 0);
	signal Q4_CLUSTERCNTB_D		: std_logic_vector(4 downto 0);
	
	signal Q1_CLUSTERCNTB_Q		: std_logic_vector(4 downto 0);
	signal Q2_CLUSTERCNTB_Q		: std_logic_vector(4 downto 0);
	signal Q3_CLUSTERCNTB_Q		: std_logic_vector(4 downto 0);
	signal Q4_CLUSTERCNTB_Q		: std_logic_vector(4 downto 0);
	
	signal CLUSTERCNTA_D		: std_logic_vector(4 downto 0);
	signal CLUSTERCNTA_Q		: std_logic_vector(4 downto 0);

	signal CLUSTERCNTB_D		: std_logic_vector(4 downto 0);
	signal CLUSTERCNTB_Q		: std_logic_vector(4 downto 0);
	
	signal CLUSTERCNTAB_D		: std_logic_vector(4 downto 0);
	signal CLUSTERCNTAB_Q		: std_logic_vector(4 downto 0);
	
	signal TRIGGER_D, TRIGGER_Q	: std_logic;
begin

	TRIGGER <= TRIGGER_Q;

	TRIGGER_D <= '1' when SYNC_Q2 = '1' and CLUSTERCNTAB_Q >= CLUSTERCNT_THR(4 downto 0) else
	             '0' when SYNC_Q2 = '1' else
	             TRIGGER_Q;

	CLUSTERCNTAB_D <= CLUSTERCNTA_Q + CLUSTERCNTB_Q;
	
	process(CLK)
	begin
		if rising_edge(CLK) then
			CLUSTERCNTAB_Q <= CLUSTERCNTAB_D;
		end if;
	end process;

	Q1_CLUSTERCNTA_D <= ('0'&Q1_CLUSTERCNT1) + ('0'&Q1_CLUSTERCNT2);
	Q2_CLUSTERCNTA_D <= ('0'&Q2_CLUSTERCNT1) + ('0'&Q2_CLUSTERCNT2);
	Q3_CLUSTERCNTA_D <= ('0'&Q3_CLUSTERCNT1) + ('0'&Q3_CLUSTERCNT2);
	Q4_CLUSTERCNTA_D <= ('0'&Q4_CLUSTERCNT1) + ('0'&Q4_CLUSTERCNT2);
	
	CLUSTERCNTA_D <= Q1_CLUSTERCNTA_Q + Q2_CLUSTERCNTA_Q + Q3_CLUSTERCNTA_Q + Q4_CLUSTERCNTA_Q;
	
	process(CLK)
	begin
		if rising_edge(CLK) then
			if SYNC_Q2 = '1' then
				Q1_CLUSTERCNTA_Q <= Q1_CLUSTERCNTA_D;
				Q2_CLUSTERCNTA_Q <= Q2_CLUSTERCNTA_D;
				Q3_CLUSTERCNTA_Q <= Q3_CLUSTERCNTA_D;
				Q4_CLUSTERCNTA_Q <= Q4_CLUSTERCNTA_D;
			end if;
		end if;
	end process;
	
	process(CLK)
	begin
		if rising_edge(CLK) then
			if SYNC = '1' then
				CLUSTERCNTA_Q <= CLUSTERCNTA_D;
			end if;
		end if;
	end process;
	
	Q1_CLUSTERCNTB_D <= stdvec_bitcnt(Q1_CLUSTERS, 5);
	Q2_CLUSTERCNTB_D <= stdvec_bitcnt(Q2_CLUSTERS, 5);
	Q3_CLUSTERCNTB_D <= stdvec_bitcnt(Q3_CLUSTERS, 5);
	Q4_CLUSTERCNTB_D <= stdvec_bitcnt(Q4_CLUSTERS, 5);
	
	CLUSTERCNTB_D <= Q1_CLUSTERCNTB_Q + Q2_CLUSTERCNTB_Q + Q3_CLUSTERCNTB_Q + Q4_CLUSTERCNTB_Q;
	
	process(CLK)
	begin
		if rising_edge(CLK) then
			if SYNC = '1' then
				Q1_CLUSTERCNTB_Q <= Q1_CLUSTERCNTB_D;
				Q2_CLUSTERCNTB_Q <= Q2_CLUSTERCNTB_D;
				Q3_CLUSTERCNTB_Q <= Q3_CLUSTERCNTB_D;
				Q4_CLUSTERCNTB_Q <= Q4_CLUSTERCNTB_D;
			end if;
		end if;
	end process;

	process(CLK)
	begin
		if rising_edge(CLK) then
			if SYNC_Q1 = '1' then
				CLUSTERCNTB_Q <= CLUSTERCNTB_D;
			end if;
		end if;
	end process;

	process(CLK, RESET)
	begin
		if RESET = '1' then
			SYNC_Q1 <= '0';
			SYNC_Q2 <= '0';
			TRIGGER_Q <= '0';
		elsif rising_edge(CLK) then
			SYNC_Q1 <= SYNC;
			SYNC_Q2 <= SYNC_Q1;
			TRIGGER_Q <= TRIGGER_D;
		end if;
	end process;
	
end RTL;