library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

use work.QTriggerPKG.all;

entity ParallelCom is
	port(
		CLK			: in std_logic;
		RESET		: in std_logic;
		SYNC		: in std_logic;
		BUSOUT		: out std_logic_vector(31 downto 0);
		QTOWERS		: in std_logic_vector(105 downto 0);
		QCLUSTERS	: in std_logic_vector(87 downto 0);
		CLUSTERCNT	: in std_logic_vector(7 downto 0);
		QSAMPLE		: out std_logic
	);
end ParallelCom;

architecture RTL of ParallelCom is
	signal COMSTATE_Q, COMSTATE_D	: std_logic_vector(2 downto 0);
	signal BUSOUT_D					: std_logic_vector(31 downto 0);
	signal MSG0_D, MSG1_D, MSG2_D	: std_logic_vector(31 downto 0);
begin

	MSG0_D <= "11110" & 
	        QTOWERS(69 downto 67) & QTOWERS(58 downto 56) & QTOWERS(47 downto 45) & QTOWERS(37 downto 35) &
	        QTOWERS(28 downto 26) & QTOWERS(20 downto 18) & QTOWERS(13 downto 11) & QTOWERS(7 downto 5) & 
	        QTOWERS(2 downto 0);
	
	MSG1_D <= "1110" & QTOWERS(105 downto 78);

	MSG2_D <= "10101" & CLUSTERCNT &
			QCLUSTERS(87 downto 79) & QCLUSTERS(69) & QCLUSTERS(59) & QCLUSTERS(49) & QCLUSTERS(39) &
			QCLUSTERS(30) & QCLUSTERS(22) & QCLUSTERS(15) & QCLUSTERS(9) & QCLUSTERS(4) & QCLUSTERS(0);

	BUSOUT_D <= MSG0_D when COMSTATE_Q(0) = '1' else
			    MSG1_D when COMSTATE_Q(1) = '1' else
			    MSG2_D when COMSTATE_Q(2) = '1' else
			    (others => '0');

	COMSTATE_D <= "001" when SYNC = '1' else COMSTATE_Q(1 downto 0) & '0';
	
	QSAMPLE <= COMSTATE_Q(1);

	process(CLK, RESET)
	begin
		if RESET = '1' then
			COMSTATE_Q <= (others => '0');
			BUSOUT <= (others => '0');
		elsif rising_edge(CLK) then
			COMSTATE_Q <= COMSTATE_D;
			BUSOUT <= BUSOUT_D(15 downto 0) & BUSOUT_D(31 downto 16);
		end if;
	end process;
	
end RTL;