library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;  -- Use OR_REDUCE function

package QTriggerPKG is

	constant V_REVISION		: std_logic_vector(15 downto 0) := x"0100";	-- 1.0
	constant V_QTRIGGERID	: std_logic_vector(6 downto 0) := "1010001";
	
	type STD_LOGIC_VEC_ARRAY4 is array(87 downto 0) of std_logic_vector(3 downto 0);
	type STD_LOGIC_VEC_ARRAY9 is array(87 downto 0) of std_logic_vector(8 downto 0);

	function stdvec_bitcnt (b:std_logic_vector; w:integer) return std_logic_vector;	
	function towerneighbors (i:integer;v:std_logic_vector) return std_logic_vector;
	function checkcluster(i:integer;w:STD_LOGIC_VEC_ARRAY4;t:std_logic_vector) return std_logic;
	function sum_9bits(I:std_logic_vector) return std_logic_vector;
	
end QTriggerPKG;

package body QTriggerPKG is

	-- sum_9bits coded for 4-LUT synthesis efficiency...
	function sum_9bits(I:std_logic_vector) return std_logic_vector is
		variable I1, I2, O	: std_logic_vector(3 downto 0);
		variable O1, O2		: std_logic_vector(2 downto 0);
	begin
		O1 := ("00"&I(0 downto 0))+("00"&I(1 downto 1))+("00"&I(2 downto 2))+("00"&I(3 downto 3));
		O2 := ("00"&I(4 downto 4))+("00"&I(5 downto 5))+("00"&I(6 downto 6))+("00"&I(7 downto 7));
		O := ("0"&O1)+("0"&O2)+("000"&I(8 downto 8));
		return O;
	end;

	function stdvec_bitcnt (b:std_logic_vector; w:integer) return std_logic_vector is
		variable result	: std_logic_vector(w-1 downto 0);
	begin
		result := (others => '0');
		for i in b'RANGE loop
			if b(i) = '1' then
				result := result + 1;
			end if;
		end loop;
		return result;
	end;
	
	function towerneighbors (i:integer;v:std_logic_vector) return std_logic_vector is
		variable result	: std_logic_vector(8 downto 0);
	begin
		case i is
			when 0 => result := "0"&"0"&"0"&v(0)&v(5)&v(1)&v(6)&v(2)&v(7);
			when 1 => result := "0"&"0"&"0"&v(1)&v(6)&v(2)&v(7)&v(3)&v(8);
			when 2 => result := "0"&"0"&"0"&v(2)&v(7)&v(3)&v(8)&v(4)&v(9);
			when 3 => result := "0"&"0"&"0"&"0"&v(3)&v(8)&v(4)&v(9)&v(10);
			when 4 => result := v(0)&v(5)&v(11)&v(1)&v(6)&v(12)&v(2)&v(7)&v(13);
			when 5 => result := v(1)&v(6)&v(12)&v(2)&v(7)&v(13)&v(3)&v(8)&v(14);
			when 6 => result := v(2)&v(7)&v(13)&v(3)&v(8)&v(14)&v(4)&v(9)&v(15);
			when 7 => result := "0"&v(3)&v(8)&v(14)&v(4)&v(9)&v(15)&v(10)&v(16);
			when 8 => result := "0"&"0"&"0"&v(4)&v(9)&v(15)&v(10)&v(16)&v(17);
			when 9 => result := v(5)&v(11)&v(18)&v(6)&v(12)&v(19)&v(7)&v(13)&v(20);
			when 10 => result := v(6)&v(12)&v(19)&v(7)&v(13)&v(20)&v(8)&v(14)&v(21);
			when 11 => result := v(7)&v(13)&v(20)&v(8)&v(14)&v(21)&v(9)&v(15)&v(22);
			when 12 => result := v(8)&v(14)&v(21)&v(9)&v(15)&v(22)&v(10)&v(16)&v(23);
			when 13 => result := "0"&v(9)&v(15)&v(22)&v(10)&v(16)&v(23)&v(17)&v(24);
			when 14 => result := "0"&"0"&"0"&v(10)&v(16)&v(23)&v(17)&v(24)&v(25);
			when 15 => result := v(11)&v(18)&v(26)&v(12)&v(19)&v(27)&v(13)&v(20)&v(28);
			when 16 => result := v(12)&v(19)&v(27)&v(13)&v(20)&v(28)&v(14)&v(21)&v(29);
			when 17 => result := v(13)&v(20)&v(28)&v(14)&v(21)&v(29)&v(15)&v(22)&v(30);
			when 18 => result := v(14)&v(21)&v(29)&v(15)&v(22)&v(30)&v(16)&v(23)&v(31);
			when 19 => result := v(15)&v(22)&v(30)&v(16)&v(23)&v(31)&v(17)&v(24)&v(32);
			when 20 => result := "0"&v(16)&v(23)&v(31)&v(17)&v(24)&v(32)&v(25)&v(33);
			when 21 => result := "0"&"0"&"0"&v(17)&v(24)&v(32)&v(25)&v(33)&v(34);
			when 22 => result := v(18)&v(26)&v(35)&v(19)&v(27)&v(36)&v(20)&v(28)&v(37);
			when 23 => result := v(19)&v(27)&v(36)&v(20)&v(28)&v(37)&v(21)&v(29)&v(38);
			when 24 => result := v(20)&v(28)&v(37)&v(21)&v(29)&v(38)&v(22)&v(30)&v(39);
			when 25 => result := v(21)&v(29)&v(38)&v(22)&v(30)&v(39)&v(23)&v(31)&v(40);
			when 26 => result := v(22)&v(30)&v(39)&v(23)&v(31)&v(40)&v(24)&v(32)&v(41);
			when 27 => result := v(23)&v(31)&v(40)&v(24)&v(32)&v(41)&v(25)&v(33)&v(42);
			when 28 => result := "0"&v(24)&v(32)&v(41)&v(25)&v(33)&v(42)&v(34)&v(43);
			when 29 => result := "0"&"0"&"0"&v(25)&v(33)&v(42)&v(34)&v(43)&v(44);
			when 30 => result := v(26)&v(35)&v(45)&v(27)&v(36)&v(46)&v(28)&v(37)&v(47);
			when 31 => result := v(27)&v(36)&v(46)&v(28)&v(37)&v(47)&v(29)&v(38)&v(48);
			when 32 => result := v(28)&v(37)&v(47)&v(29)&v(38)&v(48)&v(30)&v(39)&v(49);
			when 33 => result := v(29)&v(38)&v(48)&v(30)&v(39)&v(49)&v(31)&v(40)&v(50);
			when 34 => result := v(30)&v(39)&v(49)&v(31)&v(40)&v(50)&v(32)&v(41)&v(51);
			when 35 => result := v(31)&v(40)&v(50)&v(32)&v(41)&v(51)&v(33)&v(42)&v(52);
			when 36 => result := v(32)&v(41)&v(51)&v(33)&v(42)&v(52)&v(34)&v(43)&v(53);
			when 37 => result := "0"&v(33)&v(42)&v(52)&v(34)&v(43)&v(53)&v(44)&v(54);
			when 38 => result := "0"&"0"&"0"&v(34)&v(43)&v(53)&v(44)&v(54)&v(55);
			when 39 => result := v(35)&v(45)&v(56)&v(36)&v(46)&v(57)&v(37)&v(47)&v(58);
			when 40 => result := v(36)&v(46)&v(57)&v(37)&v(47)&v(58)&v(38)&v(48)&v(59);
			when 41 => result := v(37)&v(47)&v(58)&v(38)&v(48)&v(59)&v(39)&v(49)&v(60);
			when 42 => result := v(38)&v(48)&v(59)&v(39)&v(49)&v(60)&v(40)&v(50)&v(61);
			when 43 => result := v(39)&v(49)&v(60)&v(40)&v(50)&v(61)&v(41)&v(51)&v(62);
			when 44 => result := v(40)&v(50)&v(61)&v(41)&v(51)&v(62)&v(42)&v(52)&v(63);
			when 45 => result := v(41)&v(51)&v(62)&v(42)&v(52)&v(63)&v(43)&v(53)&v(64);
			when 46 => result := v(42)&v(52)&v(63)&v(43)&v(53)&v(64)&v(44)&v(54)&v(65);
			when 47 => result := "0"&v(43)&v(53)&v(64)&v(44)&v(54)&v(65)&v(55)&v(66);
			when 48 => result := "0"&"0"&"0"&"0"&v(44)&v(54)&v(65)&v(55)&v(66);
			when 49 => result := v(45)&v(56)&v(67)&v(46)&v(57)&v(68)&v(47)&v(58)&v(69);
			when 50 => result := v(46)&v(57)&v(68)&v(47)&v(58)&v(69)&v(48)&v(59)&v(70);
			when 51 => result := v(47)&v(58)&v(69)&v(48)&v(59)&v(70)&v(49)&v(60)&v(71);
			when 52 => result := v(48)&v(59)&v(70)&v(49)&v(60)&v(71)&v(50)&v(61)&v(72);
			when 53 => result := v(49)&v(60)&v(71)&v(50)&v(61)&v(72)&v(51)&v(62)&v(73);
			when 54 => result := v(50)&v(61)&v(72)&v(51)&v(62)&v(73)&v(52)&v(63)&v(74);
			when 55 => result := v(51)&v(62)&v(73)&v(52)&v(63)&v(74)&v(53)&v(64)&v(75);
			when 56 => result := v(52)&v(63)&v(74)&v(53)&v(64)&v(75)&v(54)&v(65)&v(76);
			when 57 => result := v(53)&v(64)&v(75)&v(54)&v(65)&v(76)&v(55)&v(66)&v(77);
			when 58 => result := "0"&"0"&"0"&v(54)&v(65)&v(76)&v(55)&v(66)&v(77);
			when 59 => result := "0"&v(56)&v(67)&v(57)&v(68)&v(78)&v(58)&v(69)&v(79);
			when 60 => result := v(57)&v(68)&v(78)&v(58)&v(69)&v(79)&v(59)&v(70)&v(80);
			when 61 => result := v(58)&v(69)&v(79)&v(59)&v(70)&v(80)&v(60)&v(71)&v(81);
			when 62 => result := v(59)&v(70)&v(80)&v(60)&v(71)&v(81)&v(61)&v(72)&v(82);
			when 63 => result := v(60)&v(71)&v(81)&v(61)&v(72)&v(82)&v(62)&v(73)&v(83);
			when 64 => result := v(61)&v(72)&v(82)&v(62)&v(73)&v(83)&v(63)&v(74)&v(84);
			when 65 => result := v(62)&v(73)&v(83)&v(63)&v(74)&v(84)&v(64)&v(75)&v(85);
			when 66 => result := v(63)&v(74)&v(84)&v(64)&v(75)&v(85)&v(65)&v(76)&v(86);
			when 67 => result := v(64)&v(75)&v(85)&v(65)&v(76)&v(86)&v(66)&v(77)&v(87);
			when 68 => result := "0"&"0"&"0"&v(65)&v(76)&v(86)&v(66)&v(77)&v(87);
			when 69 => result := "0"&"0"&"0"&v(67)&v(68)&v(78)&v(69)&v(79)&v(88);
			when 70 => result := "0"&v(68)&v(78)&v(69)&v(79)&v(88)&v(70)&v(80)&v(89);
			when 71 => result := v(69)&v(79)&v(88)&v(70)&v(80)&v(89)&v(71)&v(81)&v(90);
			when 72 => result := v(70)&v(80)&v(89)&v(71)&v(81)&v(90)&v(72)&v(82)&v(91);
			when 73 => result := v(71)&v(81)&v(90)&v(72)&v(82)&v(91)&v(73)&v(83)&v(92);
			when 74 => result := v(72)&v(82)&v(91)&v(73)&v(83)&v(92)&v(74)&v(84)&v(93);
			when 75 => result := v(73)&v(83)&v(92)&v(74)&v(84)&v(93)&v(75)&v(85)&v(94);
			when 76 => result := v(74)&v(84)&v(93)&v(75)&v(85)&v(94)&v(76)&v(86)&v(95);
			when 77 => result := v(75)&v(85)&v(94)&v(76)&v(86)&v(95)&v(77)&v(87)&v(96);
			when 78 => result := "0"&"0"&"0"&v(76)&v(86)&v(95)&v(77)&v(87)&v(96);
			when 79 => result := "0"&"0"&v(78)&v(79)&v(88)&v(97)&v(80)&v(89)&v(98);
			when 80 => result := v(79)&v(88)&v(97)&v(80)&v(89)&v(98)&v(81)&v(90)&v(99);
			when 81 => result := v(80)&v(89)&v(98)&v(81)&v(90)&v(99)&v(82)&v(91)&v(100);
			when 82 => result := v(81)&v(90)&v(99)&v(82)&v(91)&v(100)&v(83)&v(92)&v(101);
			when 83 => result := v(82)&v(91)&v(100)&v(83)&v(92)&v(101)&v(84)&v(93)&v(102);
			when 84 => result := v(83)&v(92)&v(101)&v(84)&v(93)&v(102)&v(85)&v(94)&v(103);
			when 85 => result := v(84)&v(93)&v(102)&v(85)&v(94)&v(103)&v(86)&v(95)&v(104);
			when 86 => result := v(85)&v(94)&v(103)&v(86)&v(95)&v(104)&v(87)&v(96)&v(105);
			when 87 => result := "0"&"0"&"0"&v(86)&v(95)&v(104)&v(87)&v(96)&v(105);
			when others => result := (others => '0');
		end case;
		return result;
	end;
	
	function checkcluster(i:integer;w:STD_LOGIC_VEC_ARRAY4;t:std_logic_vector) return std_logic is
		variable result	: std_logic;
	begin
		case i is
			when 0 =>
				if (w(0) > t) and (w(0) > w(1)) and (w(0) > w(4)) and (w(0) > w(5)) then
					result := '1';
				else
					result := '0';
				end if;
			when 1 =>
				if (w(1) > t) and (w(1) >= w(0)) and (w(1) > w(2)) and (w(1) >= w(4)) and (w(1) > w(5)) and (w(1) > w(6)) then
					result := '1';
				else
					result := '0';
				end if;
			when 2 =>
				if (w(2) > t) and (w(2) >= w(1)) and (w(2) > w(3)) and (w(2) >= w(5)) and (w(2) > w(6)) and (w(2) > w(7)) then
					result := '1';
				else
					result := '0';
				end if;
			when 3 =>
				if (w(3) > t) and (w(3) >= w(2)) and (w(3) >= w(6)) and (w(3) > w(7)) and (w(3) > w(8)) then
					result := '1';
				else
					result := '0';
				end if;
			when 4 =>
				if (w(4) > t) and (w(4) >= w(0)) and (w(4) > w(1)) and (w(4) > w(5)) and (w(4) > w(9)) and (w(4) > w(10)) then
					result := '1';
				else
					result := '0';
				end if;
			when 5 =>
				if (w(5) > t) and (w(5) >= w(0)) and (w(5) >= w(1)) and (w(5) > w(2)) and (w(5) >= w(4)) and (w(5) > w(6)) and (w(5) >= w(9)) and (w(5) > w(10)) and (w(5) > w(11)) then
					result := '1';
				else
					result := '0';
				end if;
			when 6 =>
				if (w(6) > t) and (w(6) >= w(1)) and (w(6) >= w(2)) and (w(6) > w(3)) and (w(6) >= w(5)) and (w(6) > w(7)) and (w(6) >= w(10)) and (w(6) > w(11)) and (w(6) > w(12)) then
					result := '1';
				else
					result := '0';
				end if;
			when 7 =>
				if (w(7) > t) and (w(7) >= w(2)) and (w(7) >= w(3)) and (w(7) >= w(6)) and (w(7) > w(8)) and (w(7) >= w(11)) and (w(7) > w(12)) and (w(7) > w(13)) then
					result := '1';
				else
					result := '0';
				end if;
			when 8 =>
				if (w(8) > t) and (w(8) >= w(3)) and (w(8) >= w(7)) and (w(8) >= w(12)) and (w(8) > w(13)) and (w(8) > w(14)) then
					result := '1';
				else
					result := '0';
				end if;
			when 9 =>
				if (w(9) > t) and (w(9) >= w(4)) and (w(9) > w(5)) and (w(9) > w(10)) and (w(9) > w(15)) and (w(9) > w(16)) then
					result := '1';
				else
					result := '0';
				end if;
			when 10 =>
				if (w(10) > t) and (w(10) >= w(4)) and (w(10) >= w(5)) and (w(10) > w(6)) and (w(10) >= w(9)) and (w(10) > w(11)) and (w(10) >= w(15)) and (w(10) > w(16)) and (w(10) > w(17)) then
					result := '1';
				else
					result := '0';
				end if;
			when 11 =>
				if (w(11) > t) and (w(11) >= w(5)) and (w(11) >= w(6)) and (w(11) > w(7)) and (w(11) >= w(10)) and (w(11) > w(12)) and (w(11) >= w(16)) and (w(11) > w(17)) and (w(11) > w(18)) then
					result := '1';
				else
					result := '0';
				end if;
			when 12 =>
				if (w(12) > t) and (w(12) >= w(6)) and (w(12) >= w(7)) and (w(12) > w(8)) and (w(12) >= w(11)) and (w(12) > w(13)) and (w(12) >= w(17)) and (w(12) > w(18)) and (w(12) > w(19)) then
					result := '1';
				else
					result := '0';
				end if;
			when 13 =>
				if (w(13) > t) and (w(13) >= w(7)) and (w(13) >= w(8)) and (w(13) >= w(12)) and (w(13) > w(14)) and (w(13) >= w(18)) and (w(13) > w(19)) and (w(13) > w(20)) then
					result := '1';
				else
					result := '0';
				end if;
			when 14 =>
				if (w(14) > t) and (w(14) >= w(8)) and (w(14) >= w(13)) and (w(14) >= w(19)) and (w(14) > w(20)) and (w(14) > w(21)) then
					result := '1';
				else
					result := '0';
				end if;
			when 15 =>
				if (w(15) > t) and (w(15) >= w(9)) and (w(15) > w(10)) and (w(15) > w(16)) and (w(15) > w(22)) and (w(15) > w(23)) then
					result := '1';
				else
					result := '0';
				end if;
			when 16 =>
				if (w(16) > t) and (w(16) >= w(9)) and (w(16) >= w(10)) and (w(16) > w(11)) and (w(16) >= w(15)) and (w(16) > w(17)) and (w(16) >= w(22)) and (w(16) > w(23)) and (w(16) > w(24)) then
					result := '1';
				else
					result := '0';
				end if;
			when 17 =>
				if (w(17) > t) and (w(17) >= w(10)) and (w(17) >= w(11)) and (w(17) > w(12)) and (w(17) >= w(16)) and (w(17) > w(18)) and (w(17) >= w(23)) and (w(17) > w(24)) and (w(17) > w(25)) then
					result := '1';
				else
					result := '0';
				end if;
			when 18 =>
				if (w(18) > t) and (w(18) >= w(11)) and (w(18) >= w(12)) and (w(18) > w(13)) and (w(18) >= w(17)) and (w(18) > w(19)) and (w(18) >= w(24)) and (w(18) > w(25)) and (w(18) > w(26)) then
					result := '1';
				else
					result := '0';
				end if;
			when 19 =>
				if (w(19) > t) and (w(19) >= w(12)) and (w(19) >= w(13)) and (w(19) > w(14)) and (w(19) >= w(18)) and (w(19) > w(20)) and (w(19) >= w(25)) and (w(19) > w(26)) and (w(19) > w(27)) then
					result := '1';
				else
					result := '0';
				end if;
			when 20 =>
				if (w(20) > t) and (w(20) >= w(13)) and (w(20) >= w(14)) and (w(20) >= w(19)) and (w(20) > w(21)) and (w(20) >= w(26)) and (w(20) > w(27)) and (w(20) > w(28)) then
					result := '1';
				else
					result := '0';
				end if;
			when 21 =>
				if (w(21) > t) and (w(21) >= w(14)) and (w(21) >= w(20)) and (w(21) >= w(27)) and (w(21) > w(28)) and (w(21) > w(29)) then
					result := '1';
				else
					result := '0';
				end if;
			when 22 =>
				if (w(22) > t) and (w(22) >= w(15)) and (w(22) > w(16)) and (w(22) > w(23)) and (w(22) > w(30)) and (w(22) > w(31)) then
					result := '1';
				else
					result := '0';
				end if;
			when 23 =>
				if (w(23) > t) and (w(23) >= w(15)) and (w(23) >= w(16)) and (w(23) > w(17)) and (w(23) >= w(22)) and (w(23) > w(24)) and (w(23) >= w(30)) and (w(23) > w(31)) and (w(23) > w(32)) then
					result := '1';
				else
					result := '0';
				end if;
			when 24 =>
				if (w(24) > t) and (w(24) >= w(16)) and (w(24) >= w(17)) and (w(24) > w(18)) and (w(24) >= w(23)) and (w(24) > w(25)) and (w(24) >= w(31)) and (w(24) > w(32)) and (w(24) > w(33)) then
					result := '1';
				else
					result := '0';
				end if;
			when 25 =>
				if (w(25) > t) and (w(25) >= w(17)) and (w(25) >= w(18)) and (w(25) > w(19)) and (w(25) >= w(24)) and (w(25) > w(26)) and (w(25) >= w(32)) and (w(25) > w(33)) and (w(25) > w(34)) then
					result := '1';
				else
					result := '0';
				end if;
			when 26 =>
				if (w(26) > t) and (w(26) >= w(18)) and (w(26) >= w(19)) and (w(26) > w(20)) and (w(26) >= w(25)) and (w(26) > w(27)) and (w(26) >= w(33)) and (w(26) > w(34)) and (w(26) > w(35)) then
					result := '1';
				else
					result := '0';
				end if;
			when 27 =>
				if (w(27) > t) and (w(27) >= w(19)) and (w(27) >= w(20)) and (w(27) > w(21)) and (w(27) >= w(26)) and (w(27) > w(28)) and (w(27) >= w(34)) and (w(27) > w(35)) and (w(27) > w(36)) then
					result := '1';
				else
					result := '0';
				end if;
			when 28 =>
				if (w(28) > t) and (w(28) >= w(20)) and (w(28) >= w(21)) and (w(28) >= w(27)) and (w(28) > w(29)) and (w(28) >= w(35)) and (w(28) > w(36)) and (w(28) > w(37)) then
					result := '1';
				else
					result := '0';
				end if;
			when 29 =>
				if (w(29) > t) and (w(29) >= w(21)) and (w(29) >= w(28)) and (w(29) >= w(36)) and (w(29) > w(37)) and (w(29) > w(38)) then
					result := '1';
				else
					result := '0';
				end if;
			when 30 =>
				if (w(30) > t) and (w(30) >= w(22)) and (w(30) > w(23)) and (w(30) > w(31)) and (w(30) > w(39)) and (w(30) > w(40)) then
					result := '1';
				else
					result := '0';
				end if;
			when 31 =>
				if (w(31) > t) and (w(31) >= w(22)) and (w(31) >= w(23)) and (w(31) > w(24)) and (w(31) >= w(30)) and (w(31) > w(32)) and (w(31) >= w(39)) and (w(31) > w(40)) and (w(31) > w(41)) then
					result := '1';
				else
					result := '0';
				end if;
			when 32 =>
				if (w(32) > t) and (w(32) >= w(23)) and (w(32) >= w(24)) and (w(32) > w(25)) and (w(32) >= w(31)) and (w(32) > w(33)) and (w(32) >= w(40)) and (w(32) > w(41)) and (w(32) > w(42)) then
					result := '1';
				else
					result := '0';
				end if;
			when 33 =>
				if (w(33) > t) and (w(33) >= w(24)) and (w(33) >= w(25)) and (w(33) > w(26)) and (w(33) >= w(32)) and (w(33) > w(34)) and (w(33) >= w(41)) and (w(33) > w(42)) and (w(33) > w(43)) then
					result := '1';
				else
					result := '0';
				end if;
			when 34 =>
				if (w(34) > t) and (w(34) >= w(25)) and (w(34) >= w(26)) and (w(34) > w(27)) and (w(34) >= w(33)) and (w(34) > w(35)) and (w(34) >= w(42)) and (w(34) > w(43)) and (w(34) > w(44)) then
					result := '1';
				else
					result := '0';
				end if;
			when 35 =>
				if (w(35) > t) and (w(35) >= w(26)) and (w(35) >= w(27)) and (w(35) > w(28)) and (w(35) >= w(34)) and (w(35) > w(36)) and (w(35) >= w(43)) and (w(35) > w(44)) and (w(35) > w(45)) then
					result := '1';
				else
					result := '0';
				end if;
			when 36 =>
				if (w(36) > t) and (w(36) >= w(27)) and (w(36) >= w(28)) and (w(36) > w(29)) and (w(36) >= w(35)) and (w(36) > w(37)) and (w(36) >= w(44)) and (w(36) > w(45)) and (w(36) > w(46)) then
					result := '1';
				else
					result := '0';
				end if;
			when 37 =>
				if (w(37) > t) and (w(37) >= w(28)) and (w(37) >= w(29)) and (w(37) >= w(36)) and (w(37) > w(38)) and (w(37) >= w(45)) and (w(37) > w(46)) and (w(37) > w(47)) then
					result := '1';
				else
					result := '0';
				end if;
			when 38 =>
				if (w(38) > t) and (w(38) >= w(29)) and (w(38) >= w(37)) and (w(38) >= w(46)) and (w(38) > w(47)) and (w(38) > w(48)) then
					result := '1';
				else
					result := '0';
				end if;
			when 39 =>
				if (w(39) > t) and (w(39) >= w(30)) and (w(39) > w(31)) and (w(39) > w(40)) and (w(39) > w(49)) and (w(39) > w(50)) then
					result := '1';
				else
					result := '0';
				end if;
			when 40 =>
				if (w(40) > t) and (w(40) >= w(30)) and (w(40) >= w(31)) and (w(40) > w(32)) and (w(40) >= w(39)) and (w(40) > w(41)) and (w(40) >= w(49)) and (w(40) > w(50)) and (w(40) > w(51)) then
					result := '1';
				else
					result := '0';
				end if;
			when 41 =>
				if (w(41) > t) and (w(41) >= w(31)) and (w(41) >= w(32)) and (w(41) > w(33)) and (w(41) >= w(40)) and (w(41) > w(42)) and (w(41) >= w(50)) and (w(41) > w(51)) and (w(41) > w(52)) then
					result := '1';
				else
					result := '0';
				end if;
			when 42 =>
				if (w(42) > t) and (w(42) >= w(32)) and (w(42) >= w(33)) and (w(42) > w(34)) and (w(42) >= w(41)) and (w(42) > w(43)) and (w(42) >= w(51)) and (w(42) > w(52)) and (w(42) > w(53)) then
					result := '1';
				else
					result := '0';
				end if;
			when 43 =>
				if (w(43) > t) and (w(43) >= w(33)) and (w(43) >= w(34)) and (w(43) > w(35)) and (w(43) >= w(42)) and (w(43) > w(44)) and (w(43) >= w(52)) and (w(43) > w(53)) and (w(43) > w(54)) then
					result := '1';
				else
					result := '0';
				end if;
			when 44 =>
				if (w(44) > t) and (w(44) >= w(34)) and (w(44) >= w(35)) and (w(44) > w(36)) and (w(44) >= w(43)) and (w(44) > w(45)) and (w(44) >= w(53)) and (w(44) > w(54)) and (w(44) > w(55)) then
					result := '1';
				else
					result := '0';
				end if;
			when 45 =>
				if (w(45) > t) and (w(45) >= w(35)) and (w(45) >= w(36)) and (w(45) > w(37)) and (w(45) >= w(44)) and (w(45) > w(46)) and (w(45) >= w(54)) and (w(45) > w(55)) and (w(45) > w(56)) then
					result := '1';
				else
					result := '0';
				end if;
			when 46 =>
				if (w(46) > t) and (w(46) >= w(36)) and (w(46) >= w(37)) and (w(46) > w(38)) and (w(46) >= w(45)) and (w(46) > w(47)) and (w(46) >= w(55)) and (w(46) > w(56)) and (w(46) > w(57)) then
					result := '1';
				else
					result := '0';
				end if;
			when 47 =>
				if (w(47) > t) and (w(47) >= w(37)) and (w(47) >= w(38)) and (w(47) >= w(46)) and (w(47) > w(48)) and (w(47) >= w(56)) and (w(47) > w(57)) and (w(47) > w(58)) then
					result := '1';
				else
					result := '0';
				end if;
			when 48 =>
				if (w(48) > t) and (w(48) >= w(38)) and (w(48) >= w(47)) and (w(48) >= w(57)) and (w(48) > w(58)) then
					result := '1';
				else
					result := '0';
				end if;
			when 49 =>
				if (w(49) > t) and (w(49) >= w(39)) and (w(49) > w(40)) and (w(49) > w(50)) and (w(49) > w(59)) and (w(49) > w(60)) then
					result := '1';
				else
					result := '0';
				end if;
			when 50 =>
				if (w(50) > t) and (w(50) >= w(39)) and (w(50) >= w(40)) and (w(50) > w(41)) and (w(50) >= w(49)) and (w(50) > w(51)) and (w(50) >= w(59)) and (w(50) > w(60)) and (w(50) > w(61)) then
					result := '1';
				else
					result := '0';
				end if;
			when 51 =>
				if (w(51) > t) and (w(51) >= w(40)) and (w(51) >= w(41)) and (w(51) > w(42)) and (w(51) >= w(50)) and (w(51) > w(52)) and (w(51) >= w(60)) and (w(51) > w(61)) and (w(51) > w(62)) then
					result := '1';
				else
					result := '0';
				end if;
			when 52 =>
				if (w(52) > t) and (w(52) >= w(41)) and (w(52) >= w(42)) and (w(52) > w(43)) and (w(52) >= w(51)) and (w(52) > w(53)) and (w(52) >= w(61)) and (w(52) > w(62)) and (w(52) > w(63)) then
					result := '1';
				else
					result := '0';
				end if;
			when 53 =>
				if (w(53) > t) and (w(53) >= w(42)) and (w(53) >= w(43)) and (w(53) > w(44)) and (w(53) >= w(52)) and (w(53) > w(54)) and (w(53) >= w(62)) and (w(53) > w(63)) and (w(53) > w(64)) then
					result := '1';
				else
					result := '0';
				end if;
			when 54 =>
				if (w(54) > t) and (w(54) >= w(43)) and (w(54) >= w(44)) and (w(54) > w(45)) and (w(54) >= w(53)) and (w(54) > w(55)) and (w(54) >= w(63)) and (w(54) > w(64)) and (w(54) > w(65)) then
					result := '1';
				else
					result := '0';
				end if;
			when 55 =>
				if (w(55) > t) and (w(55) >= w(44)) and (w(55) >= w(45)) and (w(55) > w(46)) and (w(55) >= w(54)) and (w(55) > w(56)) and (w(55) >= w(64)) and (w(55) > w(65)) and (w(55) > w(66)) then
					result := '1';
				else
					result := '0';
				end if;
			when 56 =>
				if (w(56) > t) and (w(56) >= w(45)) and (w(56) >= w(46)) and (w(56) > w(47)) and (w(56) >= w(55)) and (w(56) > w(57)) and (w(56) >= w(65)) and (w(56) > w(66)) and (w(56) > w(67)) then
					result := '1';
				else
					result := '0';
				end if;
			when 57 =>
				if (w(57) > t) and (w(57) >= w(46)) and (w(57) >= w(47)) and (w(57) > w(48)) and (w(57) >= w(56)) and (w(57) > w(58)) and (w(57) >= w(66)) and (w(57) > w(67)) and (w(57) > w(68)) then
					result := '1';
				else
					result := '0';
				end if;
			when 58 =>
				if (w(58) > t) and (w(58) >= w(47)) and (w(58) >= w(48)) and (w(58) >= w(57)) and (w(58) >= w(67)) and (w(58) > w(68)) then
					result := '1';
				else
					result := '0';
				end if;
			when 59 =>
				if (w(59) > t) and (w(59) >= w(49)) and (w(59) > w(50)) and (w(59) > w(60)) and (w(59) > w(69)) and (w(59) > w(70)) then
					result := '1';
				else
					result := '0';
				end if;
			when 60 =>
				if (w(60) > t) and (w(60) >= w(49)) and (w(60) >= w(50)) and (w(60) > w(51)) and (w(60) >= w(59)) and (w(60) > w(61)) and (w(60) >= w(69)) and (w(60) > w(70)) and (w(60) > w(71)) then
					result := '1';
				else
					result := '0';
				end if;
			when 61 =>
				if (w(61) > t) and (w(61) >= w(50)) and (w(61) >= w(51)) and (w(61) > w(52)) and (w(61) >= w(60)) and (w(61) > w(62)) and (w(61) >= w(70)) and (w(61) > w(71)) and (w(61) > w(72)) then
					result := '1';
				else
					result := '0';
				end if;
			when 62 =>
				if (w(62) > t) and (w(62) >= w(51)) and (w(62) >= w(52)) and (w(62) > w(53)) and (w(62) >= w(61)) and (w(62) > w(63)) and (w(62) >= w(71)) and (w(62) > w(72)) and (w(62) > w(73)) then
					result := '1';
				else
					result := '0';
				end if;
			when 63 =>
				if (w(63) > t) and (w(63) >= w(52)) and (w(63) >= w(53)) and (w(63) > w(54)) and (w(63) >= w(62)) and (w(63) > w(64)) and (w(63) >= w(72)) and (w(63) > w(73)) and (w(63) > w(74)) then
					result := '1';
				else
					result := '0';
				end if;
			when 64 =>
				if (w(64) > t) and (w(64) >= w(53)) and (w(64) >= w(54)) and (w(64) > w(55)) and (w(64) >= w(63)) and (w(64) > w(65)) and (w(64) >= w(73)) and (w(64) > w(74)) and (w(64) > w(75)) then
					result := '1';
				else
					result := '0';
				end if;
			when 65 =>
				if (w(65) > t) and (w(65) >= w(54)) and (w(65) >= w(55)) and (w(65) > w(56)) and (w(65) >= w(64)) and (w(65) > w(66)) and (w(65) >= w(74)) and (w(65) > w(75)) and (w(65) > w(76)) then
					result := '1';
				else
					result := '0';
				end if;
			when 66 =>
				if (w(66) > t) and (w(66) >= w(55)) and (w(66) >= w(56)) and (w(66) > w(57)) and (w(66) >= w(65)) and (w(66) > w(67)) and (w(66) >= w(75)) and (w(66) > w(76)) and (w(66) > w(77)) then
					result := '1';
				else
					result := '0';
				end if;
			when 67 =>
				if (w(67) > t) and (w(67) >= w(56)) and (w(67) >= w(57)) and (w(67) > w(58)) and (w(67) >= w(66)) and (w(67) > w(68)) and (w(67) >= w(76)) and (w(67) > w(77)) and (w(67) > w(78)) then
					result := '1';
				else
					result := '0';
				end if;
			when 68 =>
				if (w(68) > t) and (w(68) >= w(57)) and (w(68) >= w(58)) and (w(68) >= w(67)) and (w(68) >= w(77)) and (w(68) > w(78)) then
					result := '1';
				else
					result := '0';
				end if;
			when 69 =>
				if (w(69) > t) and (w(69) >= w(59)) and (w(69) > w(60)) and (w(69) > w(70)) and (w(69) > w(79)) then
					result := '1';
				else
					result := '0';
				end if;
			when 70 =>
				if (w(70) > t) and (w(70) >= w(59)) and (w(70) >= w(60)) and (w(70) > w(61)) and (w(70) >= w(69)) and (w(70) > w(71)) and (w(70) > w(79)) and (w(70) > w(80)) then
					result := '1';
				else
					result := '0';
				end if;
			when 71 =>
				if (w(71) > t) and (w(71) >= w(60)) and (w(71) >= w(61)) and (w(71) > w(62)) and (w(71) >= w(70)) and (w(71) > w(72)) and (w(71) >= w(79)) and (w(71) > w(80)) and (w(71) > w(81)) then
					result := '1';
				else
					result := '0';
				end if;
			when 72 =>
				if (w(72) > t) and (w(72) >= w(61)) and (w(72) >= w(62)) and (w(72) > w(63)) and (w(72) >= w(71)) and (w(72) > w(73)) and (w(72) >= w(80)) and (w(72) > w(81)) and (w(72) > w(82)) then
					result := '1';
				else
					result := '0';
				end if;
			when 73 =>
				if (w(73) > t) and (w(73) >= w(62)) and (w(73) >= w(63)) and (w(73) > w(64)) and (w(73) >= w(72)) and (w(73) > w(74)) and (w(73) >= w(81)) and (w(73) > w(82)) and (w(73) > w(83)) then
					result := '1';
				else
					result := '0';
				end if;
			when 74 =>
				if (w(74) > t) and (w(74) >= w(63)) and (w(74) >= w(64)) and (w(74) > w(65)) and (w(74) >= w(73)) and (w(74) > w(75)) and (w(74) >= w(82)) and (w(74) > w(83)) and (w(74) > w(84)) then
					result := '1';
				else
					result := '0';
				end if;
			when 75 =>
				if (w(75) > t) and (w(75) >= w(64)) and (w(75) >= w(65)) and (w(75) > w(66)) and (w(75) >= w(74)) and (w(75) > w(76)) and (w(75) >= w(83)) and (w(75) > w(84)) and (w(75) > w(85)) then
					result := '1';
				else
					result := '0';
				end if;
			when 76 =>
				if (w(76) > t) and (w(76) >= w(65)) and (w(76) >= w(66)) and (w(76) > w(67)) and (w(76) >= w(75)) and (w(76) > w(77)) and (w(76) >= w(84)) and (w(76) > w(85)) and (w(76) > w(86)) then
					result := '1';
				else
					result := '0';
				end if;
			when 77 =>
				if (w(77) > t) and (w(77) >= w(66)) and (w(77) >= w(67)) and (w(77) > w(68)) and (w(77) >= w(76)) and (w(77) > w(78)) and (w(77) >= w(85)) and (w(77) > w(86)) and (w(77) > w(87)) then
					result := '1';
				else
					result := '0';
				end if;
			when 78 =>
				if (w(78) > t) and (w(78) >= w(67)) and (w(78) >= w(68)) and (w(78) >= w(77)) and (w(78) >= w(86)) and (w(78) > w(87)) then
					result := '1';
				else
					result := '0';
				end if;
			when 79 =>
				if (w(79) > t) and (w(79) >= w(69)) and (w(79) >= w(70)) and (w(79) > w(71)) and (w(79) > w(80)) then
					result := '1';
				else
					result := '0';
				end if;
			when 80 =>
				if (w(80) > t) and (w(80) >= w(70)) and (w(80) >= w(71)) and (w(80) > w(72)) and (w(80) >= w(79)) and (w(80) > w(81)) then
					result := '1';
				else
					result := '0';
				end if;
			when 81 =>
				if (w(81) > t) and (w(81) >= w(71)) and (w(81) >= w(72)) and (w(81) > w(73)) and (w(81) >= w(80)) and (w(81) > w(82)) then
					result := '1';
				else
					result := '0';
				end if;
			when 82 =>
				if (w(82) > t) and (w(82) >= w(72)) and (w(82) >= w(73)) and (w(82) > w(74)) and (w(82) >= w(81)) and (w(82) > w(83)) then
					result := '1';
				else
					result := '0';
				end if;
			when 83 =>
				if (w(83) > t) and (w(83) >= w(73)) and (w(83) >= w(74)) and (w(83) > w(75)) and (w(83) >= w(82)) and (w(83) > w(84)) then
					result := '1';
				else
					result := '0';
				end if;
			when 84 =>
				if (w(84) > t) and (w(84) >= w(74)) and (w(84) >= w(75)) and (w(84) > w(76)) and (w(84) >= w(83)) and (w(84) > w(85)) then
					result := '1';
				else
					result := '0';
				end if;
			when 85 =>
				if (w(85) > t) and (w(85) >= w(75)) and (w(85) >= w(76)) and (w(85) > w(77)) and (w(85) >= w(84)) and (w(85) > w(86)) then
					result := '1';
				else
					result := '0';
				end if;
			when 86 =>
				if (w(86) > t) and (w(86) >= w(76)) and (w(86) >= w(77)) and (w(86) > w(78)) and (w(86) >= w(85)) and (w(86) > w(87)) then
					result := '1';
				else
					result := '0';
				end if;
			when 87 =>
				if (w(87) > t) and (w(87) >= w(77)) and (w(87) >= w(78)) and (w(87) >= w(86)) then
					result := '1';
				else
					result := '0';
				end if;
			when others =>
				result := '0';
		end case;
		return result;
	end;

end QTriggerPKG;